"""
Flask TDD:
https://github.com/mjhea0/flaskr-tdd

"""
import unittest
import pytest
from charpy.app import app


class FlaskTest(unittest.TestCase):

    def test_index(self):
        """ inital test. ensure flask was set up correctly """
        tester = app.test_client(self)
        app.testing = True
        response = tester.get('/', content_type='html/text')
        self.assertEqual(response.status_code, 200)

    #@pytest.mark.skip(reason="no way of currently testing this")
    def test_chart_with_data(self):
        """ inital test. ensure flask was set up correctly """
        tester = app.test_client(self)
        response = tester.get('/chart', content_type='html/text')
        self.assertEqual(response.status_code, 200)

    def test_custom_route(self):
        """ inital test. ensure flask was set up correctly """
        tester = app.test_client(self)
        response = tester.get('/hello/world/', content_type='html/text')
        self.assertEqual(response.status_code, 200)

    def test_404_not_found(self):
        """ inital test. ensure flask was set up correctly """
        tester = app.test_client(self)
        response = tester.get('/404', content_type='html/text')
        self.assertEqual(response.status_code, 404)


if __name__ == "__main__":
    unittest.main()



import os
import io
from charpy.data_import import README_RST_PATH
from charpy.data_import.converter import readme_md_to_rst
from charpy.data_import.CSVfile import CSVfile
from charpy import DATA_PATH, ROOT_PATH


def create_readme_rst():
    """
    create the readme rst
    """
    result = readme_md_to_rst()
    with open(README_RST_PATH, 'w') as f:
        f.write(result)


def get_readme_rst():
    """
    Read the README.rst file and return a string with it content
    :return: README.rst in a string
    """
    readme_path = os.path.join(ROOT_PATH, "README.rst")
    with open(readme_path, 'r') as f:
        readme = f.read()

    return readme


def get_csv_data(path, data_format=CSVfile.COLUMN, encoding='utf8'):
    """
    Return each row of the csv file

    :param encoding: encoding of the file
    :param data_format: if you want the csv data as column or row
    :param path: of the csv file to get data from
    :return:
    """
    with io.open(path, 'r', encoding=encoding, newline='') as f:
        file = CSVfile(f)

    return file.get_values(data_format)


def get_demo_data():
    """
    Get the demo data from here (3 level below root) to ./data/demo.csv

    :return: the transpose value of the demo file.
    """
    demo_file_path = os.path.join(DATA_PATH, "demo.csv")
    value = get_csv_data(demo_file_path)

    return value


if __name__ == "__main__":  # pragma: no cover
    create_readme_rst()
    # print(get_readme_rst())
    pass



# -*- coding: utf-8 -*-
"""
Use export FLASK_APP="charpy.factory:create_app()"


"""
from flask import Flask
from werkzeug.utils import find_modules, import_string
from charpy.blueprints.simple_page import simple_page


def create_app(debug=False):
    app = Flask(__name__)
    app.debug = debug

    # add your modules
    # app.register_blueprint(simple_page)
    register_blueprints(app)

    return app


def register_blueprints(app):
    """Automagically register all blueprint named bp in packages

    Check the argument 'bp' in all the modules in the folder inserted in findmodules
    then register all blueprints in the app
    """
    for name in find_modules('charpy.blueprints', recursive=True):
        mod = import_string(name)

        if hasattr(mod, 'bp'):
            app.register_blueprint(mod.bp)
    return None


if __name__ == "__main__":  # pragma: no cover
    app = create_app(debug=True)
    # print(app.blueprints)
    app.run()



import csv
from charpy.data_import.converter import transpose, string_to_date, string_to_float


class CSVfile:
    """
    CSVfile class to handle operation on an open csv file
    """
    ROW = "ROW"
    COLUMN = "COLUMN"

    def __init__(self, file, delimiter=None, has_header=None):
        """
        Initialize class with a csv file

        :param file: the open csv file
        :param dialect: optional, default ','
        :param encoding: optional, default 'utf8'
        """
        self.f = file
        self.sniffer = csv.Sniffer()
        self.delimiter = delimiter
        self.has_header = has_header
        self.data_header = None
        self.data_rows = []
        self.data_columns = []
        self.__extract_csv_data()

    def __check_delimiter(self):
        """
        Check if delimiter ',' or ';'

        :return: the dialect (delimiter to open the file)
        """
        if self.delimiter is None:
            self.delimiter = self.sniffer.sniff(self.f.readline(), [',', ';'])
            self.f.seek(0)

    def __check_header(self):
        """
        If not specified, will check if there is a header in the data, otherwise take the given data

        :return:
        """
        if self.has_header is None:
            self.has_header = self.sniffer.has_header(self.f.read(2048))
            self.f.seek(0)

    def __read(self):
        """
        Read the file and return a csv object

        :return:
        """
        self.__check_delimiter()
        reader = csv.reader(self.f, self.delimiter)
        print(reader)
        return reader

    def __extract_csv_data(self):
        """
        Extract the data from the csv file
        If it has a header the value goes to header and the rest to data,
        if not all is assigned to data

        :return: a list of rows
        """
        self.__check_header()
        raw_csv = list(self.__read())  #TODO do not handle if unicode (from ascii to utf-8)

        if self.has_header:
            self.data_header = raw_csv[0]
            self.data_rows = raw_csv[1:]
        else:
            self.data_rows = raw_csv

        self.data_columns = transpose(self.data_rows)

    def get_values(self, data_format=COLUMN):
        """ Get the data from the csv file """
        if data_format == self.ROW:
            return self.data_rows
        else:
            return self.data_columns

    def column_date_format(self, column_number, formatting='%d/%m/%Y'):
        """
        Convert the column string values to another format

        :param formatting: date format
        :param column_number: from 0 to x, which the formatting will be applied to
        """
        pass
        #map(lambda x: string_to_date(x).strftime(formatting), self.data_columns[column_number])




from flask import Blueprint, render_template, abort
from jinja2 import TemplateNotFound

simple_page = Blueprint('simple_page', __name__,
                        template_folder='templates')


@simple_page.route('/', defaults={'page': 'index'})
@simple_page.route('/pages/<page>')
def show(page):
    try:
        # return render_template('pages/%s.html' % page) # To open the template corresponding of the url
        return "The page name is: " + page
    except TemplateNotFound:
        abort(404)


@simple_page.app_errorhandler(404)  # 404 for the whole app
def page_not_found(e):
    return render_template('404.html'), 404  # render the template and return 404



from flask import Blueprint, render_template
import os
from charpy.data_import.io_import import get_csv_data
from charpy import DATA_PATH

bp = Blueprint('charpy', __name__, template_folder='templates')


@bp.route('/')
@bp.route('/charpy')
def chart_demo():
    demo_file_path = os.path.join(DATA_PATH, "pcbanking.csv")
    date, labels, values = get_csv_data(demo_file_path)
    return render_template('chart-py.html', values=values, labels=labels, type='pie')



from flask import Blueprint, render_template

bp = Blueprint('default', __name__, template_folder='templates')


@bp.app_errorhandler(404)
def page_not_found(error):
    return render_template('404.html'), 404



import unittest
import datetime
from charpy.data_import.converter import *
from tests import TEST_DATA_COLUMN, TEST_DATA_ROW


class TestConverter(unittest.TestCase):

    def test_csv_transpose(self):
        """ Make sure that transpose can transpose the csv and put it back how it was """
        self.assertEqual(TEST_DATA_ROW, transpose(TEST_DATA_COLUMN))
        self.assertEqual(TEST_DATA_COLUMN, transpose(TEST_DATA_ROW))

    def test_readme_md_to_rst_not_empty(self):
        """ check that readme to rst generate a file not empty """
        self.assertTrue(readme_md_to_rst(), True)

    def test_readme_md_converted_to_rst(self):
        """ If it is really converted the first character should not be a # """
        self.assertFalse(readme_md_to_rst()[1:] == "#")

    def test_string_to_date(self):
        """ If it is really converted the first character should not be a # """
        self.assertTrue(string_to_date("18/04/2018") == datetime.date(2018, 4, 18))

    def test_string_to_float(self):
        """ If it is really converted the first character should not be a # """
        self.assertTrue(string_to_float("1234.5678") == 1234.5678)


if __name__ == "__main__":
    unittest.main()



import unittest
from charpy.factory import create_app


class TestAppFactory(unittest.TestCase):
    def setUp(self):
        self.app = create_app()
        self.client = self.app.test_client()

    def test_index(self):
        """inital test. ensure flask was set up correctly"""
        response = self.client.get('/', content_type='html/text')
        self.assertEqual(response.status_code, 200)

    def test_404_not_found(self):
        """inital test. ensure flask was set up correctly"""
        response = self.client.get('/404', content_type='html/text')
        self.assertTrue(b'WAS NOT FOUND' in response.data)
        self.assertEqual(response.status_code, 404)


if __name__ == "__main__":
    unittest.main()



import unittest
import os
import csv
from tests import *
from charpy.data_import.CSVfile import CSVfile


class CSVfileDataTest(object):

    def setUp(self):
        """ Open the test file """
        with open("testfile.csv", 'w') as f:
            wr = csv.writer(f, quoting=csv.QUOTE_ALL)
            wr.writerows(self.FILE_DATA)
        self.file = open("testfile.csv", 'rU')

    def test_file_created(self):
        """ Make sure the test file is created and initialised well """
        self.assertTrue(list(csv.reader(self.file)) == self.FILE_DATA, "test file not created correctly")

    def test_extract_csv_data(self):
        """ Make sure the data extracted is correct """
        csv_file = CSVfile(self.file)
        self.assertTrue(csv_file.get_values(CSVfile.ROW) == TEST_DATA_ROW, "Row data extraction not correct")
        self.assertTrue(csv_file.get_values(CSVfile.COLUMN) == TEST_DATA_COLUMN, "Column data extraction not correct")

    def test_format_string_to_date(self):
        """ Check that the column is correctly formatted """
        pass

    def tearDown(self):
        """ Close the file and remove it """
        self.file.close()
        os.remove("testfile.csv")


class CSVFileTestHeader(CSVfileDataTest, unittest.TestCase):
    FILE_DATA = TEST_DATA

    def test_get_header(self):
        """ Make sur the header is populated """
        csv_file = CSVfile(self.file)
        self.assertTrue(csv_file.data_header == TEST_DATA[0], "Header not captured correctly")


class CSVFileTestNoHeader(CSVfileDataTest, unittest.TestCase):
    FILE_DATA = TEST_DATA_ROW

    def test_no_header_found(self):
        """ Make sur the header is populated """
        csv_file = CSVfile(self.file)
        self.assertFalse(csv_file.has_header, "Problem with header detection")


if __name__ == "__main__":
    unittest.main()



from dateutil.parser import parse
from charpy.data_import import README_MD_PATH


def readme_md_to_rst():
    """
    Use pypandoc to create a .rst version of the .md README

    :return: a string with the rst
    """
    try:
        import pypandoc
        result = pypandoc.convert_file(README_MD_PATH, 'rst')

    except(IOError, ImportError):
        result = open(README_MD_PATH).read()

    return result


def transpose(csv_list):
    """
    Transpose a list of rows into a list of columns

    example:
       [['x', 3], ['y', 4], ['z', 5]]

    will become:
       [['x', 'y', 'z'], [3, 4, 5]]

    :return: a transposed list of list
    """
    transposed = map(list, zip(*csv_list))

    return list(transposed)


def string_to_date(string):
    """
    The date returned is a datetime.datetime object

    No need to specify the format:
    date.year gives the year
    date.month gives the month
    date.day gives the day

    the '.date()' remove the hours.

    :param string:
    :return: the date
    """
    return parse(string).date()


def string_to_float(string):
    """
    return a float from a string

    :param string:
    :return: float
    """
    return float(string)


if __name__ == "__main__":  # pragma: no cover
    # print(readme_md_to_rst())
    string_to_date("29/10/2018")
    pass



from flask import Flask
from flask import render_template
from charpy.data_import.io_import import get_demo_data
from charpy.blueprints.simple_page import simple_page

app = Flask(__name__)
app.register_blueprint(simple_page)
# app.register_blueprint(simple_page, url_prefix='/pages') # to register blueprint at different location


@app.route("/")
def chart_demo():
    labels = ["January", "February", "March", "April", "May", "June", "July", "August"]
    values = [10, 9, 8, 7, 6, 4, 7, 8]
    colors = ["#F7464A", "#46BFBD", "#FDB45C", "#FEDCBA", "#ABCDEF", "#DDDDDD", "#ABCABC", "#BCAFED"]
    return render_template('chart.html', values=values, labels=labels, set=zip(values, labels, colors))


@app.route("/chart")
def chart_file():
    labels, values, colors = get_demo_data()

    return render_template('chart.html', values=values, labels=labels, set=zip(values, labels, colors))


@app.route("/hello/<string:name>/")
def hello(name):
    return "hello " + name + "!"


if __name__ == "__main__":  # pragma: no cover
    print(get_demo_data())
    app.run(host='127.0.0.1', port=5001)



import unittest
from charpy.data_import.io_import import *
from tests import TEST_DATA_COLUMN
import pytest


class TestIOImport(unittest.TestCase):

    #@pytest.mark.skip(reason="no way of currently testing this")
    def test_get_demo_data(self):
        """ Make sure we get the demo data from a csv file and transposed """
        self.assertEqual(TEST_DATA_COLUMN, get_demo_data())

    def test_get_readme_rst(self):
        """ check if you can retrieve the rst file and that a file is not created """
        result = get_readme_rst()
        self.assertFalse(result == "")

    def test_reate_readme_rst(self):
        """ check that readme to rst generate a file not empty """
        os.remove(README_RST_PATH)
        create_readme_rst()
        self.assertTrue(os.path.isfile(README_RST_PATH))
        self.assertTrue(os.path.getsize(README_RST_PATH) > 0)


if __name__ == "__main__":
    unittest.main()
